@@ -207,16 +207,27 @@ class Select:
         if not self.selection_functions:
             return 1
         scan_pending = False
-        for sf in self.selection_functions[:-1]:
+        for sf in self.selection_functions:
             result = sf(path)
             if result is 2:
+                # Selection function says that the path should be scanned for matching files, but keep going
+                # through the selection functions looking for a real match (0 or 1).
                 scan_pending = True
-            if result in [0, 1]:
+            elif result == 1:
+                # Selection function says file should be included.
                 return result
+            elif result == 0:
+                # Selection function says file should be excluded.
+                if scan_pending is False:
+                    return result
+                else:
+                    # scan_pending is True, meaning that a higher-priority selection function has said that this
+                    # folder should be scanned. We therefore return the scan value. We return here, rather than
+                    # below, because we don't want the exclude to be trumped by a lower-priority include.
+                    return 2
         if scan_pending:
+            # A selection function returned 2 and no other selection functions returned 0 or 1.
             return 2
-        sf = self.selection_functions[-1]
-        result = sf(path)
         if result is not None:
             return result
         else:
@@ -327,7 +338,7 @@ probably isn't what you meant.""") %
 
         include = include_default
         if line[:2] == "+ ":
-            # Check for "+ "/"- " syntax
+            # Check for "+ " or "- " syntax
             include = 1
             line = line[2:]
         elif line[:2] == "- ":
@@ -351,6 +362,7 @@ probably isn't what you meant.""") %
         # Internal. Used by ParseArgs.
         log.Notice(_("Reading globbing filelist %s") % list_name)
         separator = globals.null_separator and "\0" or "\n"
+        filelist_fp.seek(0)
         for line in filelist_fp.read().split(separator):
             line, include = self.filelist_sanitise_line(line, inc_default)
             if not line:
@@ -435,11 +447,7 @@ probably isn't what you meant.""") %
 
         def exclude_sel_func(path):
             # do not follow symbolic links when checking for file existence!
-            # path.append creates a new path object, which in turn uses setdata
-            # which in turn follows symbolic links...
-            if path.issym():
-                return None
-            if path.append(filename).exists():
+            if path.isdir() and path.append(filename).exists():
                 return 0
             else:
                 return None
@@ -465,24 +473,39 @@ probably isn't what you meant.""") %
 
         """
         # Internal. Used by glob_get_sf and unit tests.
+        match_only_dirs = False
+
+        if filename != "/" and filename[-1] == "/":
+            match_only_dirs = True
+            # Remove trailing / from directory name (unless that is the entire
+            # string)
+            filename = filename[:-1]
+
         if not filename.startswith(self.prefix):
             raise FilePrefixError(filename)
         index = tuple(filter(lambda x: x,
                              filename[len(self.prefix):].split("/")))
-        return self.glob_get_tuple_sf(index, include)
+        return self.glob_get_tuple_sf(index, include, match_only_dirs)
 
-    def glob_get_tuple_sf(self, tuple, include):
+    def glob_get_tuple_sf(self, tuple, include, match_only_dirs=False):
         """Return selection function based on tuple"""
         # Internal. Used by glob_get_filename_sf.
+
         def include_sel_func(path):
-            if (path.index == tuple[:len(path.index)] or
+            if match_only_dirs and not path.isdir():
+                # If the glob ended with a /, only match directories
+                return None
+            elif (path.index == tuple[:len(path.index)] or
                     path.index[:len(tuple)] == tuple):
                 return 1  # /foo/bar implicitly matches /foo, vice-versa
             else:
                 return None
 
         def exclude_sel_func(path):
-            if path.index[:len(tuple)] == tuple:
+            if match_only_dirs and not path.isdir():
+                # If the glob ended with a /, only match directories
+                return None
+            elif path.index[:len(tuple)] == tuple:
                 return 0  # /foo excludes /foo/bar, not vice-versa
             else:
                 return None
@@ -511,6 +534,15 @@ probably isn't what you meant.""") %
 
         """
         # Internal. Used by glob_get_sf and unit tests.
+
+        match_only_dirs = False
+
+        if glob_str != "/" and glob_str[-1] == "/":
+            match_only_dirs = True
+            # Remove trailing / from directory name (unless that is the entire
+            # string)
+            glob_str = glob_str[:-1]
+
         if glob_str.lower().startswith("ignorecase:"):
             re_comp = lambda r: re.compile(r, re.I | re.S)
             glob_str = glob_str[len("ignorecase:"):]
@@ -527,7 +559,10 @@ probably isn't what you meant.""") %
                                "|".join(self.glob_get_prefix_res(glob_str)))
 
         def include_sel_func(path):
-            if glob_comp_re.match(path.name):
+            if match_only_dirs and not path.isdir():
+                # If the glob ended with a /, only match directories
+                return None
+            elif glob_comp_re.match(path.name):
                 return 1
             elif scan_comp_re.match(path.name):
                 return 2
@@ -535,7 +570,10 @@ probably isn't what you meant.""") %
                 return None
 
         def exclude_sel_func(path):
-            if glob_comp_re.match(path.name):
+            if match_only_dirs and not path.isdir():
+                # If the glob ended with a /, only match directories
+                return None
+            elif glob_comp_re.match(path.name):
                 return 0
             else:
                 return None
